﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/13/13    Created 
//
//---------------------------------------------------------------------------

namespace System.IO
{
    using System.Linq;

    /// <summary>
    /// Defines extension methods to read and write <see cref="IBinaryFile"/> data.
    /// </summary>
    public static class IBinaryFileExtensions
    {
        /// <summary>
        /// Creates an object instance by reading from a <see cref="Stream"/>.
        /// </summary>
        /// <param name="stream">The <see cref="Stream"/> to read binary data from.</param>
        /// <returns>An object instance.</returns>
        /// <exception cref="InvalidFileType">The header does not match expected header.</exception>
        /// <exception cref="InvalidFileVersion">The file version is out of the range of possible versions.</exception>
        /// <exception cref="ArgumentNullException">The stream is null.</exception>
        /// <exception cref="NotSupportedException">The stream does not support reading.</exception>
        public static T Read<T>(this Stream stream) where T : class, IBinaryFile, new()
        {
            if (stream == null)
                throw new ArgumentNullException("stream");
            if (!stream.CanRead)
                throw new NotSupportedException("The stream does not support reading.");

            T obj = null;

            try
            {
                obj = new T(); //create object instance using parameterless constructor

                //initialize reader, set to leave stream open
                using (BinaryReader reader = new BinaryReader(input: stream, encoding: System.Text.Encoding.UTF8, leaveOpen: true))
                {
                    byte[] expectedMagicNumber = obj.MagicNumber; //get the expected header

                    if (expectedMagicNumber != null) //detect if a header should be read
                    {
                        byte[] actualMagicNumber = reader.ReadBytes(count: expectedMagicNumber.Length); //read the actual header

                        if (!actualMagicNumber.SequenceEqual(second: expectedMagicNumber)) //compare header
                            throw new InvalidFileType(expectedMagicNumber: expectedMagicNumber, actualMagicNumber: actualMagicNumber); //throw if invalid
                    }

                    if (obj.Version.HasValue) //detect if a version should be read
                    {
                        int version = reader.ReadInt32(); //read file version

                        //compare version:
                        if ((obj.MinimumVersion.HasValue && obj.MinimumVersion.Value > version) ||
                            (obj.MaximumVersion.HasValue && obj.MaximumVersion.Value < version))
                            throw new InvalidFileVersion<int>(version: version); //throw if invalid version

                        obj.Read(reader: reader, version: version); //read the file
                    }
                    else
                        obj.Read(reader: reader, version: null); //read the file
                }
            }
            catch //catch errors
            {
                //if IDisposable is implemented, cast and dispose reference
                IDisposable obj_cleanUp = obj as IDisposable;

                if (obj_cleanUp != null)
                    obj_cleanUp.Dispose();

                throw; //rethrow exception
            }

            return obj; //return instance
        }

        /// <summary>
        /// Writes the object data to the specified <see cref="Stream"/>.
        /// </summary>
        /// <param name="stream">The <see cref="Stream"/> to write data to.</param>
        /// <exception cref="ArgumentNullException">The stream is null.</exception>
        /// <exception cref="NotSupportedException">The stream does not support writing.</exception>
        public static void Write<T>(this T obj, Stream stream) where T : class, IBinaryFile, new()
        {
            if (stream == null)
                throw new ArgumentNullException(paramName: "stream");
            if (!stream.CanWrite)
                throw new NotSupportedException(message: "The stream does not support writing.");

            //initializes the writer, leaves the stream open after disposing
            using (BinaryWriter writer = new BinaryWriter(output: stream, encoding: System.Text.Encoding.UTF8, leaveOpen: true))
            {
                byte[] actualHeader = obj.MagicNumber; //get the header

                if (actualHeader != null) //determine if a header exists
                    writer.Write(buffer: actualHeader); //write the header

                if (obj.Version.HasValue) //determine if a version exists
                    writer.Write(value: obj.Version.Value); //write the version

                obj.Write(writer: writer); //write data
            }
        }
    }
}
