﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/13/13    Created 
//
//---------------------------------------------------------------------------

namespace System.IO
{
    using System.Runtime.Serialization;
    using System.Security.Permissions;

    /// <summary>
    /// The InvalidFileType exception is thrown when a file header is invalid.
    /// </summary>
    [Serializable]
    public sealed class InvalidFileType : IOException
    {
        #region Constants
        private const string INVALID_FILE_MESSAGE = "Attempted to read an invalid file type in file: '{0}'. Expected header: '{1}', actual header: '{2}'.";
        private const string INVALID_STREAM_MESSAGE = "Attempted to read an invalid file type. Expected header: '{0}', actual header: '{1}'."; 
        #endregion

        #region Properties
        /// <summary>
        /// Gets the file which has an invalid header.
        /// </summary>
        public string FileName { get; private set; }
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes the InvalidFileType with a file.
        /// </summary>
        /// <param name="fileName">The file that has thrown the exception.</param>
        public InvalidFileType(string fileName)
            : base(FormatByteMessage(fileName, null, null))
        {
            FileName = fileName;
        }

        /// <summary>
        /// Initializes the InvalidFileType with a expected/actual header.
        /// </summary>
        /// <param name="expectedMagicNumber">The expected file header.</param>
        /// <param name="actualMagicNumber">The actual file header.</param>
        public InvalidFileType(byte[] expectedMagicNumber, byte[] actualMagicNumber)
            : base(FormatByteMessage(string.Empty, expectedMagicNumber, actualMagicNumber))
        {
        }

        /// <summary>
        /// Initializes the InvalidFileType with a expected/actual header.
        /// </summary>
        /// <param name="expectedMagicNumber">The expected file header.</param>
        /// <param name="actualMagicNumber">The actual file header.</param>
        public InvalidFileType(string expectedMagicNumber, string actualMagicNumber)
            : base(FormatStringMessage(string.Empty, expectedMagicNumber, actualMagicNumber))
        {
        }

        /// <summary>
        /// Initializes the InvalidFileType with a file and expected/actual header.
        /// </summary>
        /// <param name="fileName">The file that has thrown the exception.</param>
        /// <param name="expectedMagicNumber">The expected file header.</param>
        /// <param name="actualMagicNumber">The actual file header.</param>
        public InvalidFileType(string fileName, byte[] expectedMagicNumber, byte[] actualMagicNumber)
            : base(FormatByteMessage(fileName, expectedMagicNumber, actualMagicNumber))
        {
            FileName = fileName;
        }

        /// <summary>
        /// Initializes the InvalidFileType with a file and expected/actual header.
        /// </summary>
        /// <param name="fileName">The file that has thrown the exception.</param>
        /// <param name="expectedMagicNumber">The expected file header.</param>
        /// <param name="actualMagicNumber">The actual file header.</param>
        public InvalidFileType(string fileName, string expectedMagicNumber, string actualMagicNumber)
            : base(FormatStringMessage(fileName, expectedMagicNumber, actualMagicNumber))
        {
            FileName = fileName;
        }
        #endregion

        #region Serialization
        /// <summary>
        /// Initializes a new instance of the InvalidFileType class with the specified serialization and context information.
        /// </summary>
        /// <param name="info">The data for serializing or deserializing the object.</param>
        /// <param name="context">The source and destination for the object.</param>
        private InvalidFileType(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            FileName = info.GetString("FileName");
        }
        /// <summary>
        /// Sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
        /// </summary>
        /// <param name="info"> The System.Runtime.Serialization.SerializationInfo that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The System.Runtime.Serialization.StreamingContext that contains contextual information about the source or destination.</param>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
            info.AddValue("FileName", FileName, typeof(string));
        }
        #endregion

        #region Formating
        private static string FormatByteMessage(string fileName, byte[] expectedMagicNumber, byte[] actualMagicNumber)
        {
            if (string.IsNullOrEmpty(fileName))
                return string.Format(INVALID_STREAM_MESSAGE, FormatBytes(expectedMagicNumber), FormatBytes(actualMagicNumber));
            else
                return string.Format(INVALID_FILE_MESSAGE, fileName, FormatBytes(expectedMagicNumber), FormatBytes(actualMagicNumber));
        }

        private static string FormatStringMessage(string fileName, string expectedMagicNumber, string actualMagicNumber)
        {
            if (string.IsNullOrEmpty(fileName))
                return string.Format(INVALID_STREAM_MESSAGE, expectedMagicNumber ?? "null", actualMagicNumber ?? "null");
            else
                return string.Format(INVALID_FILE_MESSAGE, fileName, expectedMagicNumber ?? "null", actualMagicNumber ?? "null");
        }

        private static string FormatBytes(byte[] b)
        {
            if (b == null)
                return "null";

            string result = string.Empty;

            for (int i = 0; i < b.Length; i++)
            {
                if (result != string.Empty)
                    result += ", ";

                result += b[i].ToString();
            }

            return result;
        }
        #endregion
    }
}