﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/13/13    Created 
//
//---------------------------------------------------------------------------

namespace System.IO
{
    using System.Runtime.Serialization;
    using System.Security.Permissions;

    /// <summary>
    /// The InvalidFileVersion class is an exception that is thrown when the actual version of a file is not within the minimum and maximum expected versions.
    /// </summary>
    /// <typeparam name="T">The type of version.</typeparam>
    [Serializable]
    public sealed class InvalidFileVersion<T> : IOException
    {
        #region Properties
        /// <summary>
        /// Get the version.
        /// </summary> 
        public T Version { get; private set; }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the InvalidFileVersion class with a version.
        /// </summary>
        /// <param name="Version">The file version.</param>
        public InvalidFileVersion(T version)
            : base(GetMessage(version))
        {
            Version = version;
        }
        #endregion

        #region Formatting
        /// <summary>
        /// Returns a string for the Message property.
        /// </summary>
        /// <param name="version">The file version.</param>
        /// <returns>A string detailing the exception.</returns>
        private static string GetMessage(T version)
        {
            return String.Format("The file version '{0}' is out of the range of possible versions.", version);
        }
        #endregion

        #region Serialization
        /// <summary>
        /// Initializes a new instance of the InvalidFileVersion class with the specified serialization and context information.
        /// </summary>
        /// <param name="info">The data for serializing or deserializing the object.</param>
        /// <param name="context">The source and destination for the object.</param>
        private InvalidFileVersion(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            Version = (T)info.GetValue("Version", typeof(T));
        }

        /// <summary>
        /// Sets the System.Runtime.Serialization.SerializationInfo with information about the exception.
        /// </summary>
        /// <param name="info"> The System.Runtime.Serialization.SerializationInfo that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The System.Runtime.Serialization.StreamingContext that contains contextual information about the source or destination.</param>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Version", Version, typeof(T));
            base.GetObjectData(info, context);
        }
        #endregion
    }
}
